@@ -298,8 +298,10 @@ int main(int argc, char **argv){
 void *thread_update_pui(void *arg){
 
     while (done == 0) {
-	if(!opt.quiet)
+	if(!opt.quiet){
 	    update_pui(&prog, copied, block_id, done);
+	    sleep(opt.fresh);
+	}
     }
     pthread_exit("exit");
 }
